/* fonctionsGLUT.c
 * Programme montrant quelques fonctions de l'OpenGL Utility Toolkit (GLUT)
 * (cration de popup-menus, timer, objets gomtriques, affichage de fontes)
 * Touches:
 *	ESC - Quitter
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 20 aot 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Caractristiques de la fentre */
#define TITLE "FonctionsGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Variable contenant le temps au moment de l'affichage de la
 * dernire image, cet variable permet de baser les
 * mouvements sur le temps et non pas sur la vitesse d'affichage
 * de votre ordinateur */
static int g_timeAtLastFrame = 0;

/* Objets gomtriques */
#define CUBE 1001
#define SPHERE 1002
#define CONE 1003
#define TORUS 1004
#define DODECAHEDRON 1005
#define OCTAHEDRON 1006
#define TETRAHEDRON 1007
#define ICOSAHEDRON 1008
#define TEAPOT 1009

/* Mode de rendu */
#define SOLID 10
#define WIRE 11

/* Antialiasing */
#define ANTIALIAS 12

/* Caractristiques de la lumires */
static GLfloat diffuseLight[] = { 1.0f, 1.0f, 1.0f, 1.0f };	
static GLfloat positionLight[] = { 0.5f, 2.0f, -0.8f, 1.0f };
/* Caractristiques des matriaux */
static GLfloat specularMat[] = { 0.5f, 0.5f, 0.5f, 1.0f };
static GLfloat shinyMat = 50.0f;

/* Variables pour la rotation des objets */
static float rotX = 0.0;
static float rotY = 0.0;
static float rotZ = 0.0;

/* Objet gomtrique courant */
static int object = CUBE;
/* Chane de caractre reprsentant l'objet courant */
static char objCh[15] = "Cube";
/* Mode de rendu courant */
static bool modeSolid = true;

/* Chane de caractre representant le timer */
static char timerCh[50] = "0";	

/* Prototypes */
void init(void);
void glutDisplay(void);
void glutIdle(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void glutTimer(int value);
void glutMenu(int value);
void createMenus(void);
void terminateApplication(void);
void draw_string_stroke(void *font, const char* string);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
	glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
	glutCreateWindow(TITLE);	
	init();
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutTimerFunc(100,glutTimer,100);
	createMenus();
	glutMainLoop();
	return 0;
}

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tESC - Quitter\n");
	
	g_timeAtLastFrame = glutGet(GLUT_ELAPSED_TIME);
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // amliore les calculs de perspective
	
	/* Paramtrage des lumires */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, diffuseLight);
	glLightfv(GL_LIGHT0, GL_POSITION, positionLight);
	
	/* Paramtrage du matriau */
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularMat);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shinyMat);
	
	// Pour les proprits ambiante et diffuse de la matire
	// on utilise les mmes proprits que la couleur active
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// Placement de l'angle de vue
	glTranslatef(-0.5, 0.5, -5.0);
	
	// couleur orange
	glColor3f(0.9, 0.3, 0.0);
	
	glPushMatrix();
	glRotatef(rotY, 1.0, 0.0, 0.0);
	glRotatef(rotZ, 0.0, 1.0, 0.0);
	glRotatef(rotX, 0.0, 0.0, 1.0);
	
	switch (object) {
		case CUBE: modeSolid ? glutSolidCube(2.5) : glutWireCube(2.5); 
			strcpy(objCh, "Cube");
			break;
		
		case SPHERE: modeSolid ? glutSolidSphere(1.75, 50, 50) : glutWireSphere(1.75, 50, 50);
			strcpy(objCh, "Sphere");
			break;
		
		case CONE: modeSolid ? glutSolidCone(1.5, 2.25, 50, 30) : glutWireCone(1.5, 2.25, 50, 30); 
			strcpy(objCh, "Cone");
			break;
		
		case TORUS: modeSolid ? glutSolidTorus(0.75, 1.25 ,50, 50) : glutWireTorus(0.75, 1.25 ,50, 50);
			strcpy(objCh, "Torus");
			break;
		
		case DODECAHEDRON: modeSolid ? glutSolidDodecahedron() : glutWireDodecahedron(); 
			strcpy(objCh, "Dodecahedron");
			break;
		
		case OCTAHEDRON: modeSolid ? glutSolidOctahedron() : glutWireOctahedron(); 
			strcpy(objCh, "Octahedron");
			break;
		
		case TETRAHEDRON: modeSolid ? glutSolidTetrahedron() : glutWireTetrahedron(); 
			strcpy(objCh, "Tetrahedron");
			break;
		
		case ICOSAHEDRON: modeSolid ? glutSolidIcosahedron() : glutWireIcosahedron(); 
			strcpy(objCh, "Icosahedron");
			break;
		
		case TEAPOT: modeSolid ? glutSolidTeapot(1.3) : glutWireTeapot(1.2); 
			strcpy(objCh, "Teapot");
			break;
	}
	glPopMatrix();

	// on dsactive la gestion de la lumire
	// pour l'criture du texte
	glDisable(GL_LIGHTING);
  	glColor3f(0.7f, 0.7f, 0.8f);
	
	// criture du temps du timer
	glPushMatrix();
  		glTranslatef(1.5, -3.1, 0.0f);
  		glScalef(0.0015f, 0.0015f, 1.0f);
  		draw_string_stroke(GLUT_STROKE_MONO_ROMAN, timerCh);
  	glPopMatrix();
	
	// criture du nom de l'objet
	glPushMatrix();
  		glTranslatef(1.5, -2.9, 0.0f);
  		glScalef(0.0015f, 0.0015f, 1.0f);
  		draw_string_stroke(GLUT_STROKE_MONO_ROMAN, objCh);
  	glPopMatrix();
	
	// criture du mode de rendu
	glPushMatrix();
  		glTranslatef(1.5, -2.7, 0.0f);
  		glScalef(0.0015f, 0.0015f, 1.0f);
  		draw_string_stroke(GLUT_STROKE_MONO_ROMAN, modeSolid ? "Solid" : "Wireframe");
  	glPopMatrix();
	
	glEnable(GL_LIGHTING);
	
	glFlush();
	glutSwapBuffers();
}

/* Fonction excut lorsque le programme n'a plus rien  faire */
void glutIdle(void)
{
	int currentTime = glutGet(GLUT_ELAPSED_TIME);
	int frameInterval = currentTime - g_timeAtLastFrame; // Temps coul entre l'affichage de 2 images
	g_timeAtLastFrame = currentTime;
	
	// On incrmente les angles de rotation
	rotX += (float)frameInterval / 35;
	rotY += (float)frameInterval / 35;
	rotZ += (float)frameInterval / 30;
	
	if (rotX >= 360.0f)
		rotX = 0.0f;
	if (rotY >= 360.0f)
		rotY = 0.0f;
	if (rotZ >= 360.0f)
		rotZ = 0.0f;
	
	// on redemande l'affichage de la scne
	glutPostRedisplay();
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(65.0, (GLfloat)w/(GLfloat)h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
	}
}

/* Fonction appel lorsque le timer est termin */
// toutes les 100 ms on ractualise le timer
void glutTimer(int value) 
{
	// compteur du temps
	static double c = 0.0;
	
	// on rengage le timer pour 100ms
	glutTimerFunc(100,glutTimer,100);
	c += 0.100;
	
	// on transforme le double c en chane de caractres
	gcvt(c, 7, timerCh);
	
	if (c > 3600.0)
		c = 0.0;
}

/* Fonction grant les actions sur les menus */
void glutMenu(int value)
{
	static bool antialias = false;
	
	if (value >= CUBE && value <= TEAPOT)
		object = value;
	else switch(value) {
		case ESC:		// exit
			terminateApplication();
			break;
		case SOLID: modeSolid = true; break;
		case WIRE: modeSolid = false; break;
		case ANTIALIAS: 
			antialias = !antialias;
			if (antialias) {
				/* Anti-aliasing des lignes */
				glEnable(GL_LINE_SMOOTH);
        			glEnable(GL_BLEND);
			}
			else {
				glDisable(GL_LINE_SMOOTH);
        			glDisable(GL_BLEND);
			}
			break;
	}
}

/* Cration des diffrents menus */
void createMenus(void)
{
	int mnuMain = glutCreateMenu(glutMenu);
	int mnuObject = glutCreateMenu(glutMenu);
	int mnuMode = glutCreateMenu(glutMenu);
	
	// menu mode
	glutSetMenu(mnuMode);
	glutAddMenuEntry("Solid", SOLID);
	glutAddMenuEntry("Wireframe", WIRE);
	
	// menu object
	glutSetMenu(mnuObject);
	glutAddMenuEntry("Cube", CUBE);
	glutAddMenuEntry("Sphere", SPHERE);
	glutAddMenuEntry("Cone", CONE);
	glutAddMenuEntry("Torus", TORUS);
	glutAddMenuEntry("Dodecahedron", DODECAHEDRON);
	glutAddMenuEntry("Octahedron", OCTAHEDRON);
	glutAddMenuEntry("Tetrahedron", TETRAHEDRON);
	glutAddMenuEntry("Icosahedron", ICOSAHEDRON);
	glutAddMenuEntry("Teapot", TEAPOT);	
	
	// menu principal
	glutSetMenu(mnuMain);
	glutAddSubMenu("Mode", mnuMode);
	glutAddSubMenu("Geometric Object", mnuObject);
	glutAddMenuEntry("Antialiasing on/off", ANTIALIAS);
	glutAddMenuEntry("Exit (esc)", ESC);
	
	glutAttachMenu(GLUT_RIGHT_BUTTON);	
}

/* Termine l'application */
void terminateApplication(void)
{
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}

/* Affiche une chane de caractre */
void draw_string_stroke(void *font, const char* string) 
{
	while (*string)
    		glutStrokeCharacter(font, *string++);
}



